#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'program_information/constants/regex.rb'
# =========================================================================== #
module ProgramInformation

  # ========================================================================= #
  # === REGEX_FOR_TWO_HYPHENS
  #
  # This is valid for input such as 'lynx-2.8.8-rel2' or
  # 'gobject-introspection-1.53.4' or 'subtle-0.11.3224-xi' or
  # 'dvd+rw-tools-7.1' or 'Archive-Zip-1.46' or 'btrfs-progs-v4.12.1'
  # or 'color-tools-1.3.0' or 'libedit-20170329-3.1' or
  # 'ncbi-blast-2.6.0+' or 'b43-fwcutter-015' or 'wcd-6.0.1-beta3'
  # or 'oxygen-icons5-5.37.0' or 'btrfs-progs-v4.13.2' or
  # 'Archive-Zip-1.46' or 'wcd-6.0.1-beta3' or 'openobex-1.7.1-Source'
  # or 'youtube-dl-2019.06.27'.
  #
  # See:
  #
  #   https://rubular.com/r/u9Yy3qcxrlp5uG
  #
  # ========================================================================= #
  REGEX_FOR_TWO_HYPHENS = 
    /([A-Za-z+]{0,12}-?[a-z0-9]{0,9}-?[A-Za-z]{0,14}\d{0,4}-?[A-Za-z0-9]{0,12})-(\d{0,2}\.?[a-z]{0,1}\d{0,2}\.?[a-z]{0,1}\d{0,4}[.|-]?\d{0,8}[A-Za-z]{0,9}\d?\+?)$/

  # ========================================================================= #
  # === REGEX_FOR_THREE_OR_MORE_HYPHENS
  #
  # This is for input such as 'akonadi-calendar-tools-17.08.1' or
  # 'boost-build-2014-10' or 'xf86-video-i810-1.7.4' or
  # or 'xfce4-statusnotifier-plugin-0.2.0' or 'mime-types-data-3.2016.0521'
  # or 'XML-LibXML-Simple-0.98' or 'kismet-2016-07-R1.tar.xz' or
  # 'polkit-kde-agent-1-5.15.2' or 'xf86-video-amd-2.7.7.7'
  # or 'xf86-video-s3virge-1.11.0'.
  #
  # See: https://rubular.com/r/lCrdU01HE7MRV4
  # ========================================================================= #
  REGEX_FOR_THREE_OR_MORE_HYPHENS =
    /^([A-Za-z]{0,9}\d{0,2}-?[A-Za-z]{0,12}\d{0,2}-?[a-z]{0,9}\d{0,1}-?[A-Za-z]{0,8}-?1?\d{0,3}[a-z]{0,3})-(\d{0,4}\.?\d{0,4}\.?-?[A-Z]{0,1}\d{0,4}\.?\d{0,4})$/

  # ========================================================================= #
  # === REGEX_FOR_INPUT_HAVING_ONLY_DOTS
  #
  # This is valid for strange input such as 'expect5.45.3' or
  # 'sendmail.8.15.2'. These have no '-' and no '_'.
  # ========================================================================= #
  REGEX_FOR_INPUT_HAVING_ONLY_DOTS = # See: http://rubular.com/r/KVePR7ShX3
    /([a-z]{1,10})\.?(\d+\.?\d{1,2}\.?\d{0,2})/

  # ========================================================================= #
  # === REGEX_FOR_BOOST_LIKE_INPUT
  #
  # Valid input should be 'ncbi_cxx-12_0_0' or 'icu4c-61_1'.
  #
  # See: http://rubular.com/r/CyNqJbCMR2
  # ========================================================================= #
  REGEX_FOR_BOOST_LIKE_INPUT =
    /([a-z]{0,3}\d{0,1}[a-z]{0,5}_?[a-z]{0,3})([-|_]{1})(\d{1,2}_\d{1,2}_?\d*)/

  # ========================================================================= #
  # === REGEX_FOR_ACL_LIKE_INPUT
  #
  # Valid input should be 'acl_2.2.47-1' or 'e_dbus-1.7.10' or
  # 'wpa_supplicant-2.6' or 'SDL2_gfx-1.0.1' or 'libatomic_ops-7.4.2'
  # or 'mod_python-11.10.2017' or 'mini_portile2-2.3.0.gem'.
  #
  # See: http://rubular.com/r/XDdBNWxipp
  # ========================================================================= #
  REGEX_FOR_ACL_LIKE_INPUT =
    /([A-Za-z]{0,10}[0-9]{0,1}_?[0-9]{0,1}[a-z]{1,10}[0-9]{0,1})(_|-)([0-9]{0,2}\.?[0-9]{0,2}\.?[0-9]{0,2}-?[0-9]{0,2}_?[0-9]{0,4})/

  # ========================================================================= #
  # === REGEX_FOR_INPUT_CONTAINING_TWO_HYPHENS_AND_ONE_UNDERSCORE
  #
  # This is input for programs such as 'ntfs-3g_ntfsprogs-2017.3.23'.
  #
  # See: http://rubular.com/r/15pAuHKxt5
  # ========================================================================= #
  REGEX_FOR_INPUT_CONTAINING_TWO_HYPHENS_AND_ONE_UNDERSCORE =
    /([a-z]{1}.+)(-)(\d{1,4}\.\d\.\d{1,2})/

end