#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ProgramInformation::ProgramInformation
#
# The main class will reside here.
#
# This class can provide us with the proper name, and the proper version
# for any given program.
#
# We simply take the given input and use three components for it:
#
#   (1) real_short_name
#   (2) short_name
#   (3) program_version
#
# Note that real_short_name depends on short_name. It simply is short_name
# with all instances of the '_' character removed. It is thus a dependent
# variable.
#
# The three numbers above also correspond to respective @instance_variables,
# so in other words we have @real_short_name, @short_name and
# @program_version.
#
# The input can be a bit "unclean". The class will try to get the "best"
# result out of any given input, no matter how unclean it is.
#
# It is recommended to use this class when wanting to determine the
# short_name and the program version of any given program.
# =========================================================================== #
# require 'program_information/program_information.rb'
# =========================================================================== #
require 'remove_file_suffix'
require 'program_information/project/project_base_directory.rb'
require 'program_information/toplevel_methods/e.rb'
require 'program_information/toplevel_methods/return_proper_pipe_token_to_this_input.rb'
require 'program_information/toplevel_methods/remove_archive_type.rb'
require 'program_information/constants/constants.rb'
require 'program_information/version/version.rb'
require 'program_information/initialize.rb'
require 'program_information/menu.rb'
require 'program_information/reset.rb'
require 'program_information/report.rb'
require 'program_information/run.rb'

module ProgramInformation

class ProgramInformation # === ProgramInformation::ProgramInformation

  # ========================================================================= #
  # === return_proper_pipe_token_to_this_input
  # ========================================================================= #
  def return_proper_pipe_token_to_this_input(i)
    ::ProgramInformation.return_proper_pipe_token_to_this_input(i)
  end

  # ========================================================================= #
  # === remove_file_suffix
  #
  # Delegate towards class RemoveFileSuffix. This is easier to handle
  # Archive.
  # ========================================================================= #
  def remove_file_suffix(i)
    RemoveFileSuffix[i.to_s]
  end

  # ========================================================================= #
  # === do_not_replace_plus
  # ========================================================================= #
  def do_not_replace_plus
    @replace_plus_with_long_name = false
  end; alias do_not_replace_anything do_not_replace_plus # === do_not_replace_anything

  # ========================================================================= #
  # === set_program_version
  #
  # Use this method whenever you wish to assign to the @program_version
  # variable.
  #
  # The input to this method may include stuff such as ".tar.xz", which
  # should be chopped off, as it is not part of the program version.
  #
  # Keep in mind that we may also return nil.
  #
  # Until March 2015 we also replaced all '_' with '.' characters,
  # but input such as "ncbi_cxx--12_0_0.tar.gz" would break as a
  # consequence, so this was disabled again.
  # ========================================================================= #
  def set_program_version(i)
    i = remove_file_suffix(i) unless i.nil? # Disallow archive-types here.
    # ======================================================================= #
    # The following check is not good at all because we may have input
    # such as "1-0.112.0". Polkitqt has this, for instance.
    # Thus, the following line was disabled on Feb 2016.
    # ======================================================================= #
    # if i.include? '-' # In this case, we use only the last part.
    #   i = i.split('-').last
    # end
    # ======================================================================= #
    # And if result is actually an empty String, but the original input
    # has included at the least one number, such as in "3.3.6-pl1",
    # then we will simply use that instead.
    # ======================================================================= #
    if i.is_a?(String) and i.empty? and
       @input =~ /\d+/ # <- If the input has at the least one Number.
      i = @input.dup
    end
    @program_version = i
  end; alias set_version set_program_version # === set_version

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @be_verbose
  end

  # ========================================================================= #
  # === remove_plus_token?
  # ========================================================================= #
  def remove_plus_token?
    @remove_plus_token
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === return_array
  #
  # This method will return a composite, 3-member Array that will hold the
  # short_name, the real_short_name, and the program_version.
  # ========================================================================= #
  def return_array
    [ short_name?, real_short_name?, program_version? ]
  end; alias array? return_array # === array?

  # ========================================================================= #
  # === remove_plus_and_minus?
  # ========================================================================= #
  def remove_plus_and_minus?
    @remove_plus_and_minus
  end; alias remove_plus_and_minus remove_plus_and_minus? # === remove_plus_and_minus

  # ========================================================================= #
  # === apostroph?
  # ========================================================================= #
  def apostroph?
    APOSTROPH
  end

  # ========================================================================= #
  # === replace_plus_with_long_name?
  # ========================================================================= #
  def replace_plus_with_long_name?
    @replace_plus_with_long_name
  end

  # ========================================================================= #
  # === short_name_and_version?
  #
  # This method returns an aggregated @short_name+'-'+@program_version
  # string.
  # ========================================================================= #
  def short_name_and_version?
    "#{short_name?}-#{program_version?}"
  end; alias short_name_and_version short_name_and_version? # === short_name_and_version

  # ========================================================================= #
  # === set_short_name_then_version
  #
  # This method will combine two other methods.
  # ========================================================================= #       
  def set_short_name_then_version(a, b)
    set_short_name(a)
    set_program_version(b)
  end; alias set_name_then_version set_short_name_then_version # === set_name_then_version

  # ========================================================================= #
  # === return_real_short_name_and_version
  #
  # We delegate to the module-method here.
  # ========================================================================= #
  def return_real_short_name_and_version(i)
    ::ProgramInformation.return_real_short_name_and_version(i)
  end

  # ========================================================================= #
  # === return_name_and_version
  #
  # This method will return an array with two elements:
  #
  #   (1) The first element is the name of the program in question.
  #   (2) The second element is the version of the program in question.
  #
  # The input can be rather tricky and complicated.
  #
  # Consider the following examples:
  #
  #     'libunwind-1.0-rc1'
  #     'htop-1.0.0'
  #     'WebKit-r93670'
  #     'xfce4-dev-tools-4.8.0'
  #     'e_dbus-1.7.10.tar.gz'
  #     'boost_1_54_0.tar.bz2'
  #     'short-mime-1.0.0-rc1'
  #
  # This can all be valid input.
  #
  # Which are the key-assumptions that we can do here?
  #
  # I think the best assumption we can do is to find the first NUMBER in
  # the given string, and then group the rest of the data according to
  # this NUMBER. This will not work in the case of something such as
  # WebKit-r93, as there is a 'r' character before that number.
  # So we will have to make a second assumption: if the character preceding
  # the first found integer is NOT a '-' character, we will instead split
  # on that character instead. This will however lead to a problem for 
  # something like "xfce4-dev-tools-4.8.0". So, in case there is a '-'
  # character immediately after the pos variable, we assume we have such
  # a special case.
  #
  # On Sep 2011, I came up with a far simpler idea. We simply have to
  # find the most important '-' and then split accordingly. As long as
  # we can identify the most important '-' correctly, so long will
  # everything else "just work". This also did not work out, so I
  # switched to a regex instead. Since then it has worked very fine.
  #
  # Take input such as:
  #
  #   "font-bitstream-75dpi-1.0.0"
  #
  # Note though that this approach does not work with input like:
  #   "CrownCutlass-Alpha1.4"
  # unless we assume that alpha is a part of the version,
  # which is possibly correct.
  # ========================================================================= #
  def return_name_and_version(i = nil)
    if i
      determine_the_three_main_variables(i)
    end
    # ======================================================================= #
    # We must consider what we wish to do about the '_' characters. We could
    # remove them, but certain programs such as e_dbus may require it as 
    # part of their name. Also, Version entries such as '1_2_3' will become
    # '123' rather than '1.2.3'. Thus, on Jan 2014 I have disabled the
    # following line.
    # ======================================================================= #
    # i.delete!('_') if i.include? '_'
    return [
      @short_name, @program_version
    ]
  end; alias rnav return_name_and_version # === rnav

  # ========================================================================= #
  # === set_shall_we_downcase
  # ========================================================================= #
  def set_shall_we_downcase(
      i = SHALL_WE_DOWNCASE
    )
    @shall_we_downcase = i
  end

  # ========================================================================= #
  # === do_not_downcase
  # ========================================================================= #
  def do_not_downcase
    @shall_we_downcase = false
  end
  
  # ========================================================================= #
  # === set_be_verbose
  #
  # Set to report in a verbose way here.
  # ========================================================================= #
  def set_be_verbose(i = BE_VERBOSE)
    case i
    when :be_verbose
      i = true
    end
    @be_verbose = i
  end

  # ========================================================================= #
  # === determine_the_three_main_variables
  #
  # This method will determine the three main instance variables:
  #
  #   - @short_name
  #   - @real_short_name
  #   - @program_version
  #
  # We must be careful, as the input may not include any '-' or '_' token,
  # and then the regex would fail. Hence, the behaviour of this method
  # will depend on whether the input includes a '-' or whether it
  # does not.
  #
  # Note that presently, set_input() will modify the given input and
  # ensure that there will be a '-' token, even in input that does
  # not have any '-' or '_' such as in 'sendmail.8.15.2'.
  # ========================================================================= #
  def determine_the_three_main_variables(i = @input)
    if i
      if i.include? '|'
        splitted        = i.split('|')
        short_name      = splitted.first.to_s.dup
        program_version = splitted.last.to_s.dup
        set_short_name(short_name)
        set_program_version(program_version)
      else
        opn; e "The input `#{i}` does not include a | character."
      end
    end
  end; alias determine_name_and_program_version determine_the_three_main_variables # === determine_name_and_program_version

  # ========================================================================= #
  # === set_real_short_name
  #
  # The instance variable @real_short_name is not allowed to include any
  # '-' characters. It may also not include any '_' characters.
  #
  # The latter could be found as part of program names such as "SDL_image".
  # The @short_name variant may include these '_' tokens.
  #
  # Note that as of September 2017, we also disallow '.' tokens, such as
  # may occur accidentally in input like "sendmail.5.8.2".
  # ========================================================================= #
  def set_real_short_name(i = @short_name)
    i = i.to_s.dup.delete('-_.').rstrip
    @real_short_name = i
  end

  # ========================================================================= #
  # === set_short_name
  #
  # Use this method whenever you wish to assign to the @short_name variable.
  #
  # This variable is allowed to keep '-' and '_' entries, but some
  # settings may also modify this.
  # ========================================================================= #
  def set_short_name(i)
    i = i.dup
    # ======================================================================= #
    # First, consider getting rid of '+' and '-' tokens in the
    # @short_name variant of the program name.
    # For this to work, @remove_plus_and_minus has to be set to true.
    # ======================================================================= #
    if remove_plus_and_minus?
      i.delete!('+') if i.include? '+'
      i.delete!('-') if i.include? '-'
    end
    # ======================================================================= #
    # === Get rid of '+' tokens, if they are included.
    # ======================================================================= #
    if remove_plus_token?
      i.delete!('+') if i.include? '+'
    # ======================================================================= #
    # === Alternatively, replace '+' with the string 'plus' should it
    # exist there.
    # ======================================================================= #
    elsif replace_plus_with_long_name?
      i.gsub!(/\+/, 'plus') if i.include? '+'
    end
    @short_name = i
    set_real_short_name(i) # We will also sync towards @real_short_name.
  end; alias set_name         set_short_name # === set_name
       alias set_program_name set_short_name # === set_program_name

  # ========================================================================= #
  # === remove_source_like_strings
  # ========================================================================= #
  def remove_source_like_strings(i)
    # ======================================================================= #
    # i.gsub!(/--/,'-') if i.include? '--' # Replace '--' with '-'.
    # ^^^ May not be good, depending on the input.
    # ======================================================================= #
    i.sub!(/-src/,'')     if i.include?  '-src' # Get rid of -src entries.
    i.sub!(/\.source/,'') if i.include?  '.source'
    i.sub!(/-source$/,'') if i.end_with? '-source'
    return i
  end

  # ========================================================================= #
  # === remove_archive_type
  # ========================================================================= #
  def remove_archive_type(i)
    ::ProgramInformation.remove_archive_type(i)
    return i
  end

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    ::ProgramInformation.e(i)
  end

  # ========================================================================= #
  # === real_short_name?
  #
  # Keep in mind that @real_short_name does not have any '-' tokens.
  # ========================================================================= #
  def real_short_name?
    @real_short_name
  end; alias real_name?           real_short_name? # === real_name?
       alias real_program_name?   real_short_name? # === real_program_name?
       alias real_short_name      real_short_name? # === real_short_name
       alias name_of_the_program? real_short_name? # === name_of_the_program?

  # ========================================================================= #
  # === short_name?
  #
  # Reader method for the @short_name instance variable.
  #
  # short_name is an alias to @short_name.
  # ========================================================================= #
  def short_name?
    @short_name
  end; alias program_name             short_name? # === program_name
       alias program_name?            short_name? # === program_name?      
       alias name                     short_name? # === name
       alias name?                    short_name? # === name?
       alias return_short_name        short_name? # === return_short_name
       alias short_name               short_name? # === short_name
       alias return_name              short_name? # === return_name
       alias first                    short_name? # === first
       alias return_program_real_name short_name? # === return_program_real_name

  # ========================================================================= #
  # === program_version?
  #
  # Getter method for the @program_version variable.
  # ========================================================================= #
  def program_version?
    @program_version
  end; alias program_version         program_version? # === program_version
       alias version                 program_version? # === version
       alias version?                program_version? # === version?
       alias last?                   program_version? # === last?
       alias last                    program_version? # === last?
       alias version_of_the_program? program_version? # === version_of_the_program?

  # ========================================================================= #
  # === set_input
  #
  # This method will, after performing some sanitize-operations, assign
  # to the @input variable.
  # ========================================================================= #
  def set_input(i)
    if i.is_a? Array
      menu(i)
      i = i.join(' ').strip
    end
    i = i.to_s.dup if i.frozen?
    i = File.basename(i)
    # ======================================================================= #
    # We .squeeze away '--' tokens.
    # ======================================================================= #
    i.squeeze!('-')
    i.strip!
    # ======================================================================= #
    # Next, some simple replacements without any method.
    # ======================================================================= #
    i.sub!(/-paco/,'')        if i.include? '-paco'
    i.sub!(/\/Programs\//,'') if i.include? '/Programs'
    # ======================================================================= #
    # The following must not conflict with i.e. "pkgconfig", hence why we
    # also include a '/' character.
    # ======================================================================= #
    i.gsub!(/\/pkg/,'') if i.include? '/pkg'
    # ======================================================================= #
    # i.gsub!(/\//,'-') # Controversial. Disabled for now as of Jan 2014.
    # ======================================================================= #
    # We don't want either { or }.
    # ======================================================================= #
    i.delete!('{') if i.include? '{'
    i.delete!('}') if i.include? '}'
    # ======================================================================= #
    # As of July 2016, we get rid of all tokens that follow a '#' character.
    # We also will get rid of the '#' as well. Must come before we call
    # remove_archive_type().
    # ======================================================================= #
    if i.include? '#'
      i = i[0...i.index('#')]
    end
    i = remove_archive_type(i)
    i = remove_source_like_strings(i)
    # ======================================================================= #
    # Next, we correct for input such as "nana%201.5.4". This one has no
    # '-' token and no '_' token, but a '%' token. This '%' token will be
    # replaced.
    # ======================================================================= #
    if i.include? '%'
      unless i.include?('_') or i.include?('-')
        i.tr!('%','-')
      end
    end
    # ======================================================================= #
    # Leading '-' will be removed.
    # ======================================================================= #
    i[0,1] = '' if i.start_with? '-'
    # ======================================================================= #
    # === Handle downcasing of the given input
    #
    # This will consider downcasing the input. The reason as to why this is
    # necessary is so that input such as "Libgnome" can become "libgnome".
    # ======================================================================= #
    i.downcase! if @shall_we_downcase
    # ======================================================================= #
    # === Handle '+' tokens
    # ======================================================================= #
    if REMOVE_PLUS_TOKEN
      i.delete!('+') if i.include? '+'
    end
    i = return_proper_pipe_token_to_this_input(i)
    @input = i
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # To invoke this method, try:
  #
  #   pinfo --show-help
  #
  # ========================================================================= #
  def show_help
    e
    e 'This class is able to split a given input (a string) into'
    e 'the inferred program-name, and associated program version '\
      'of said string.'
    e
    e 'For example, if you have this input string:'
    e
    e '  gobject-introspection-1.52.1'
    e
    e 'then this class can split this into these constituents:'
    e
    e '  @real_short_name: gobjectintrospection'
    e '  @short_name:      gobject-introspection'
    e '  @program_version: 1.52.1'
    e
    e 'This is especially useful for other projects that require this'
    e 'functionality, such as the rbt gem.'
    e
  end

end

# =========================================================================== #
# === ProgramInformation[]
# =========================================================================== #
def self.[](i)
  ::ProgramInformation::ProgramInformation.new(i)
end; self.instance_eval { alias new      [] } # === ProgramInformation.new
     self.instance_eval { alias parse    [] } # === ProgramInformation.parse
     self.instance_eval { alias combined [] } # === ProgramInformation.combined

end

if __FILE__ == $PROGRAM_NAME
  ProgramInformation::ProgramInformation.new(ARGV)
end # pinfo /Users/x/SRC/gobjectintrospection/gobject-introspection-1.52.1.tar.xz