#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'program_information/toplevel_methods.rb'
# =========================================================================== #
require 'program_information/toplevel_methods/opnn.rb'
require 'program_information/toplevel_methods/e.rb'
require 'program_information/toplevel_methods/return_short_name.rb'
require 'program_information/program_information.rb'

module ProgramInformation

  # ========================================================================= #
  # === ProgramInformation.return_program_version
  #
  # This module-method will return the program version of a given input.
  #
  # For example, if the input to this method is
  # 'gobject-introspection-1.52.1.tar.xz', then this method will
  # return the String '1.52.1'. 
  #
  # We must also handle the case when the version passed to us is incomplete
  # or wrong, such as "2.1.0-source". In that case, the proper program
  # version would be 2.1.0.
  #
  # Usage examples, using the shorter variant instead:
  #
  #   ProgramInformation.return_version('/opt/ffmpeg-2.3.tar.xz')             # => "2.3"
  #   ProgramInformation.return_version 'ophcrack-3.6.0'                      # => "3.6.0"
  #   ProgramInformation.return_version 'gobject-introspection-1.52.1.tar.xz' # => "1.52.1"
  #
  # ========================================================================= #
  def self.return_program_version(of_this_program = '')
    if of_this_program.nil? or of_this_program.empty?
      opnn; e 'Please provide some input to the method '\
              'ProgramInformation.return_program_version.'
    else
      _ = ::ProgramInformation::ProgramInformation.new(of_this_program)
      _.program_version?
    end 
  end; self.instance_eval { alias return_version    return_program_version } # === ProgramInformation.return_version
       self.instance_eval { alias return_version_of return_program_version } # === ProgramInformation.return_version_of

  # ========================================================================= #
  # === ProgramInformation.return_real_short_name
  #
  # This method will return the real short name.
  #
  # Usage example:
  #
  #   ProgramInformation.return_real_short_name('iso-codes-3.50.tar.xz')
  # 
  # The above would return "isocodes".
  # ========================================================================= #
  def self.return_real_short_name(of_this_program = '')
    if of_this_program.nil? or of_this_program.empty?
      opnn; e 'Please provide some input to the method '\
              'ProgramInformation.return_real_short_name.'
    else
      _ = ::ProgramInformation::ProgramInformation.new(of_this_program)
      _.real_short_name?
    end
  end; self.instance_eval { alias real_short_name     return_real_short_name } # === ProgramInformation.real_short_name
       self.instance_eval { alias return_real_name    return_real_short_name } # === ProgramInformation.return_real_name
       self.instance_eval { alias return_program_name return_real_short_name } # === ProgramInformation.return_program_name
       self.instance_eval { alias program_name?       return_real_short_name } # === ProgramInformation.program_name?

  # ========================================================================= #
  # === ProgramInformation.return_short_name_and_program_version
  #
  # This will return an Array with two entries - the short name,
  # and the program version.
  # ========================================================================= #
  def self.return_short_name_and_program_version(
      i,
      replace_input = true
    )
    case replace_input
    when :do_not_modify_input
      replace_input = false
    end
    _ = ::ProgramInformation::ProgramInformation.new(i, :dont_run_yet)
    _.do_not_replace_anything unless replace_input
    _.run
    # ======================================================================= #
    # Assemble the Array here.
    # ======================================================================= #
    array = [ _.short_name?, _.program_version? ]
    return array
  end; self.instance_eval { alias return_short_name_and_version return_short_name_and_program_version } # === ProgramInformation.return_short_name_and__version

  # ========================================================================= #
  # === ProgramInformation.return_array_name_and_version
  #
  # This will return the short_name and the version, as Array.
  #
  # Usage example:
  #
  #   ProgramInformation::ProgramInformation.return_array_name_and_version(
  #     "font-sony-misc-1.0.3") => ["font-sony-misc", "1.0.3"]
  #
  # ========================================================================= #
  def self.return_array_name_and_version(i)
    array = [
      ::ProgramInformation.return_short_name(i),
      ::ProgramInformation.return_program_version(i)
    ]
    return array
  end; self.instance_eval { alias return_array_program_name_and_program_version return_array_name_and_version } # === ProgramInformation.return_array_program_name_and_program_version

  # ========================================================================= #
  # === ProgramInformation.return_real_short_name_and_version
  #
  # This method will return two results, in Array form:
  #
  #   The @real_short_name and
  #   the @program_version
  #
  # The optional second argument can be used to downcase the program
  # name that is returned.
  # ========================================================================= #
  def self.return_real_short_name_and_version(
      i, optional_downcase = false
    )
    case optional_downcase
    when :downcase
      optional_downcase = true
    end
    real_short_name = ::ProgramInformation.return_real_short_name(i)
    if optional_downcase
      real_short_name.downcase!
    end
    program_version = ::ProgramInformation.return_program_version(i)
    return [
      real_short_name, program_version
    ]
  end; self.instance_eval { alias return_real_name_and_version                  return_real_short_name_and_version } # === ProgramInformation.return_real_name_and_version
       self.instance_eval { alias return_real_short_name_and_program_version    return_real_short_name_and_version } # === ProgramInformation.return_real_short_name_and_program_version
       self.instance_eval { alias return_name_and_version                       return_real_short_name_and_version } # === ProgramInformation.return_name_and_version
       self.instance_eval { alias return_name_and_version_of                    return_real_short_name_and_version } # === ProgramInformation.return_name_and_version_of

end

if __FILE__ == $PROGRAM_NAME
  # ========================================================================= #
  # Some testing code follows next:
  # ========================================================================= #
  if ARGV.empty?
    _ = 'gobject-introspection-1.52.1.tar.xz'
    p ProgramInformation.return_program_version(_)
    p ProgramInformation.return_short_name(_)
    p ProgramInformation.return_real_short_name(_)
  else
    p ProgramInformation.return_program_version(ARGV.first)
    p ProgramInformation.return_short_name(ARGV.first)
    p ProgramInformation.return_real_short_name(ARGV.first)
  end
end # rb toplevel_methods.rb /Users/x/SRC/gobjectintrospection/gobject-introspection-1.52.1.tar.xz